module DsEeprom (
	clk,
	reset,
	
	UartRxData,
	UartRxClear,
	UartRxFull,
	
	UartTxFull,
	UartTxData,
	UartTxClear,
	
	
	SRAM_DQ,
	SRAM_ADDR,
	SRAM_LB_N,
	SRAM_UB_N,
	SRAM_CE_N,
	SRAM_OE_N,
	SRAM_WE_N,	
	
	DSCARD_EEPROMCS,
	DSCARD_EEPROMCLK,
	DSCARD_EEPROMIN,
	DSCARD_EEPROMOUT
);


	input				clk;
	input				reset;
	
	output	[7:0]		UartRxData;
	input				UartRxClear;
	output				UartRxFull;
	
	input				UartTxFull;
	input	[7:0]		UartTxData;
	output				UartTxClear;
		

	input				DSCARD_EEPROMCS;
	input				DSCARD_EEPROMCLK;
	input				DSCARD_EEPROMIN;
	output				DSCARD_EEPROMOUT;
	
	inout   [15:0]		SRAM_DQ;
	output  [17:0] 		SRAM_ADDR;
	output         	 	SRAM_LB_N;
	output        	  	SRAM_UB_N;
	output         		SRAM_CE_N;
	output        	  	SRAM_OE_N;
	output        	  	SRAM_WE_N;		
	
	wire	[7:0]		DataIn;
	wire	[7:0]		DataOut;
	wire				DataReady;	
	wire				ChipSelect;
	
	wire	[1:0]		SramByteEnable;
	wire				SramChipSelect;
	wire	[15:0]		SramRdData;

	
	reg		[7:0]		OutBuffer;
	reg		[7:0]		InBuffer;
	reg		[4:0]		State;
	reg					DataWasReady;
	
	reg		[15:0]		SramWrData;
	reg					SramWr;
	reg		[17:0]		SramAddress;
	
	reg					SRWD;
	reg					BP1;
	reg					BP0;
	reg					WEL;
	reg					WIP;
	reg		[15:0]		Address;
	
	reg		[7:0]		UartRxData;
	reg					UartRxFull;
	reg					UartRxWasFull;
	reg					UartTxClear;
	reg					UartTxWasFull;

	
	assign				DataIn = InBuffer;
	
	assign				SramChipSelect = 1'b1;
	assign				SramByteEnable = 2'b11;

	
	parameter			STATE_IDLE			= 5'h0;
	parameter			STATE_WRSR			= 5'h1;
	parameter			STATE_WRIT			= 5'h2;
	parameter			STATE_READ			= 5'h3;
	parameter			STATE_WRDI			= 5'h4;
	parameter			STATE_RDSR			= 5'h5;
	parameter			STATE_WREN			= 5'h6;
	parameter			STATE_UARTWR		= 5'hE;
	parameter			STATE_UARTRD		= 5'hF;
	
	parameter			STATE_WRSRDATA		= 5'h10;	
	parameter			STATE_WRITADDR0		= 5'h11;
	parameter			STATE_WRITADDR1		= 5'h12;
	parameter			STATE_WRITDATA		= 5'h13;
	parameter			STATE_READADDR0		= 5'h14;
	parameter			STATE_READADDR1		= 5'h15;
	parameter			STATE_READDATA		= 5'h16;
	parameter			STATE_UARTWRDATA	= 5'h17;
	parameter			STATE_UARTRDSTAT	= 5'h18;
	parameter			STATE_UARTRDDATA	= 5'h19;
	
	
	
	always @(posedge clk)
	begin
		if (reset)
		begin
			OutBuffer <= 8'h00;
			InBuffer <= 8'h00;
			State <= STATE_IDLE;
			DataWasReady <= 1'b0;
			SramWrData <= 16'h0000;
			SramWr <= 1'b0;
			SRWD <= 1'b0;
			BP1 <= 1'b0;
			BP0 <= 1'b0;
			WEL <= 1'b0;
			WIP <= 1'b0;
			Address <= 16'h0000;
			SramAddress <= 16'h0000;
			
			UartRxFull <= 1'b0;
			UartRxWasFull <= 1'b0;
			UartRxData <= 8'h00;
			
			UartTxWasFull <= 1'b0;
			UartTxClear <= 1'b0;
		end
		else
		begin
			UartRxData <= ((~UartRxWasFull) & DataWasReady & (State == STATE_UARTWRDATA))? OutBuffer : UartRxData;
			UartRxFull <= UartRxClear? 1'b0 : ((~UartRxWasFull) & DataWasReady & (State == STATE_UARTWRDATA))? 1'b1 : UartRxFull;
			UartRxWasFull <= (DataWasReady & (State == STATE_UARTWR))? UartRxFull : UartRxWasFull;
			
			UartTxClear <= UartTxWasFull & DataWasReady & (State == STATE_UARTRDDATA);		   
			UartTxWasFull <= (DataWasReady & (State == STATE_UARTRD))? UartTxFull : UartTxWasFull;
			

			OutBuffer <= DataReady? DataOut : OutBuffer;
			DataWasReady <= DataReady;
			State <= (ChipSelect)? STATE_IDLE :
					 (DataReady & (State == STATE_IDLE))? {1'b0, DataOut[3:0]} : 
					 (DataReady & (State == STATE_READ))? STATE_READADDR0 :
					 (DataReady & (State == STATE_READADDR0))? STATE_READADDR1 :
					 (DataReady & (State == STATE_READADDR1))? STATE_READDATA :
					 (DataReady & (State == STATE_WRIT))? STATE_WRITADDR0 :
 					 (DataReady & (State == STATE_WRITADDR0))? STATE_WRITADDR1 :
					 (DataReady & (State == STATE_WRITADDR1))? STATE_WRITDATA :
					 (DataReady & (State == STATE_WRSR))? STATE_WRSRDATA :
					 (DataReady & (State == STATE_UARTWR))? STATE_UARTWRDATA :
					 (DataReady & (State == STATE_UARTRD))? STATE_UARTRDSTAT :
					 (DataReady & (State == STATE_UARTRDSTAT))? STATE_UARTRDDATA :
					 State;
			
			WEL <= (DataWasReady & (State == STATE_WREN))? 1'b1 :
				   (DataWasReady & (State == STATE_WRDI))? 1'b0 :
				   WEL;
			BP0 <= (DataWasReady & (State == STATE_WRSRDATA))? OutBuffer[2] : BP0;
			BP1 <= (DataWasReady & (State == STATE_WRSRDATA))? OutBuffer[3] : BP1;
			SRWD <= (DataWasReady & (State == STATE_WRSRDATA))? OutBuffer[7] : SRWD;
			SramAddress <= {2'b0, Address};
			SramWr <= DataWasReady & (State == STATE_WRITDATA);
			SramWrData <= {8'b0, OutBuffer};
			Address <= (DataWasReady & (State == STATE_READADDR0))? {OutBuffer, Address[7:0]} :
					   (DataWasReady & (State == STATE_WRITADDR0))? {OutBuffer, Address[7:0]} :
					   (DataWasReady & (State == STATE_READADDR1))? {Address[15:8], OutBuffer} :
					   (DataWasReady & (State == STATE_WRITADDR1))? {Address[15:8], OutBuffer} :
					   (DataWasReady & (State == STATE_READDATA))? Address + 16'h1 :
					   (DataWasReady & (State == STATE_WRITDATA))? Address + 16'h1 :
					   Address;
			
			
			
			InBuffer <= (ChipSelect)? 8'hFF :
						(State == STATE_RDSR)? {SRWD, 3'b0, BP1, BP0, WEL, WIP} :
						(State == STATE_READADDR1)? SramRdData[7:0] :
						(State == STATE_READDATA)? SramRdData[7:0] :
						(State == STATE_UARTWR)? {7'b0, UartRxWasFull} :
						(State == STATE_UARTRD)? {7'b0, UartTxWasFull} :
						(State == STATE_UARTRDSTAT)? UartTxData :
						InBuffer;
		end
	end
	
	
	SramController sramcontroller 
	(
		.clk(clk),
		.reset(reset),

		.address(SramAddress),
		.byteenable(SramByteEnable),
		.chipselect(SramChipSelect),
		.read(~SramWr),
		.write(SramWr),
		.writedata(SramWrData),
		.readdata(SramRdData),
		
		.SRAM_DQ(SRAM_DQ),
		.SRAM_ADDR(SRAM_ADDR),
		.SRAM_LB_N(SRAM_LB_N),
		.SRAM_UB_N(SRAM_UB_N),
		.SRAM_CE_N(SRAM_CE_N),
		.SRAM_OE_N(SRAM_OE_N),
		.SRAM_WE_N(SRAM_WE_N)
	);	

	DsSpi dsspi
	(
		.clk(clk),
		.reset(reset),
		
		.DataIn(DataIn),
		.DataOut(DataOut),
		.DataReady(DataReady),
		.ChipSelect(ChipSelect),

		.DSCARD_EEPROMCS(DSCARD_EEPROMCS),
		.DSCARD_EEPROMCLK(DSCARD_EEPROMCLK),
		.DSCARD_EEPROMIN(DSCARD_EEPROMIN),
		.DSCARD_EEPROMOUT(DSCARD_EEPROMOUT)
	);

endmodule
